using System;
using Microsoft.SPOT;

/// <summary>
/// The CmdDimmer class provides a dimmer functionality. The current functionality:
/// 1) A short push on the button will set or clear the output channel.
/// 2) When the users sets the output, and keeps pushing the button, the output will start to dim.
/// </summary>
class CmdDimmer : SwitchButton
{
    // declarations
    private int _outputDAC;                     // the assigned DAC channel
    private int _cycleTime = -1;                // the time of the cycle
    //private bool dimmerIsActivated = false;     // keeps the binary state of the dimmer (on/off)
    private int dimDownCycles = 0;              // counter to keep how many down dimmings were made already
    private static int _DIMTIME = 10;           // total time the dimmer should dim


    /// <summary>
    /// This parameter allows you to set the time in which the dimmer is dimmed from max to zero.
    /// </summary>
    public static int DIMTIME
    {
        get { return _DIMTIME; }
        set { _DIMTIME = value; }
    }


    /// <summary>
    /// The constructor CmdDimmer is called upon the creation of the class.
    /// </summary>
    /// <param name="inputChannel">the input channel to monitor</param>
    /// <param name="outputDAC">The output DAC channel to control</param>
    public CmdDimmer(int inputChannel, int outputDAC)
    {
        _outputDAC = outputDAC;
        _cycleTime = (DIMTIME / 64);
        createPermanentInstance(this, inputChannel);
    }



    /// <summary>
    /// buttonDown() is executed when the dimbutton has just been pressed.
    /// </summary>
    public override void buttonDown() 
    { 
        if (dimmerIsActivated())  // helper variable to keep the binary state of the dimmer (on/off).
        {
            //dimmerIsActivated = false;
            k8000().clearDACchannel(_outputDAC);
        }
        else
        {
            //dimmerIsActivated = true;
            k8000().setDACchannel(_outputDAC);
            dimDownCycles = 0;
        }
    }



    private Boolean dimmerIsActivated()
    {
        return k8000().DAC[_outputDAC] != 0;
    }




    /// <summary>
    /// buttonPressed is executed when we keep pressing the button. The dimmer has to be dimmed IF
    /// it is activated.
    /// </summary>
    /// <param name="time_ms">the total time we kept pushing the button</param>
    public override void buttonPressed(int time_ms) 
    {
        if (dimmerIsActivated())
        {   // the dimmer is activated. Make sure the dimming time is respected.
            if (time_ms > (dimDownCycles * _cycleTime))
            {
                dimDownCycles++;
                int currentDAC = k8000().DAC[_outputDAC];
                currentDAC = currentDAC - 1;
                if (currentDAC < 0)
                {
                    currentDAC = 0;
                    //dimmerIsActivated = false;  // prevents overflow of the currentDAC integer variable.
                }

                k8000().outputDACchannel(_outputDAC, currentDAC);
            }
        }
    }
}

