using System;
using Microsoft.SPOT;
using System.IO;
using Microsoft.SPOT.IO;
using GHIElectronics.NETMF.IO;
using System.Text;



/// <summary>
/// the CmdLogger provides logging functionalities.
/// </summary>
class CmdLogger : SwitchButton
{
    // define the variables
    SD myCard;
    private int[] _channelsToLog;
    private int[] channelStates;
    private String textToWrite = "";
    private int textSize = 0;
    private long previousCheck;


    /// <summary>
    /// The constructor CmdToggle is called upon the creation of the class.
    /// </summary>
    public CmdLogger(params int[] channelsToLog)
    {
        previousCheck = DateTime.Now.Ticks;
        _channelsToLog = channelsToLog;
        channelStates = new int[channelsToLog.Length];
        myCard = new SD();
        createPermanentInstance(this);        
    }



    /// <summary>
    /// monitorChannels() will monitor all selected channels. Changes are appended to the data log.
    /// </summary>
    public void monitorChannels()
    {
        // loop through all channels.
        for (int i = 0; i < _channelsToLog.Length; i++)
        {
            if (k8000().IO[_channelsToLog[i]] != channelStates[i])
            {
                try
                {
                    if (textSize < 255)
                    {
                        textToWrite += DateTime.Now.ToString() + ";I/O " + _channelsToLog[i] + ";" + k8000().IO[_channelsToLog[i]] + "\r\n";
                        textSize++;
                    }
                }
                catch (Exception e) 
                {
                    Debug.Print("Error while adding log: " + e.Message);
                }

                channelStates[i] = k8000().IO[_channelsToLog[i]];
            }
        }


    }


    /// <summary>
    /// This function is cycled once per cycle. Here the class will check if any of the
    /// monitored channels is changed. The changed channels are then appended to the
    /// textTowrite variable.
    /// </summary>
    public override void buttonCycle() 
    {
        // the channels are monitored.
        monitorChannels();

        // now every 60 seconds we will check if any data was changed.
        TimeSpan elapsedSpan = new TimeSpan(DateTime.Now.Ticks - previousCheck);
        if (elapsedSpan.Seconds < 0) previousCheck = DateTime.Now.Ticks;

        if (elapsedSpan.Minutes > 0)
        {   // store the current date/time for the next cycle
            previousCheck = DateTime.Now.Ticks;
            

            try
            {   
                if (myCard.hasSDcard())
                {   // there is an SD card connected. Check if it is mounted?
                    if (!myCard.isMounted())
                    {   // mount card first. The actual writing to the card is not done until the next cycle.
                        Debug.Print("SD card detected but not mounted. Attempting to mount the SD card...");
                        myCard.mountCard();                        
                    }
                    else
                    {   // Card is mount. Write the data, if there is any available.
                        // TODO: it might be necessary to do the actual writing in a second thread.
                        Debug.Print("card inserted and mounted");

                        if (textToWrite != "")
                        {
                            myCard.openFile("homeLog.csv");
                            myCard.writeFile(textToWrite);
                            myCard.closeFile();
                            textToWrite = "";
                            textSize = 0;
                            Debug.Print("Data sucessfully appended to the SD card");
                        }
                    }
                }
                else
                {   // there is no card connected. Do nothing.
                    textToWrite = "";
                    textSize = 0;
                    myCard.Dispose();
                   Debug.Print("No SD card...");
                }

            }
            catch (Exception e)
            {
               Debug.Print("Error! " + e.Message);
            }
        }
    }



    /// <summary>
    /// The class SD contains all the functionalities to write things on the FEZ's SD card.
    /// </summary>
    internal sealed class SD
    {
        // define the variables
        private PersistentStorage card;
        private FileStream FileHandle;
        private bool _isMounted = false;


        /// <summary>
        /// hasSDcard() determines if an SD card is inserted into the FEZ. The function will also make sure
        /// sure the PersistentStorage object is created when doing so.
        /// </summary>
        /// <returns></returns>
        public bool hasSDcard()
        {
            bool hasSD = PersistentStorage.DetectSDCard();

            try
            {
                if (hasSD == true)
                {
                    if (card == null)
                        card = new PersistentStorage("SD");
                }
            }
            catch (Exception e)
            {
                Debug.Print("Error while reading the SD card: " + e.Message);
                hasSD = false;
            }

            return hasSD;
        }



        /// <summary>
        /// mountCard will mount the SD card on the device. It returns a boolean wherether the 
        /// action succeeded and the volume has a valid formatting.
        /// </summary>
        /// <returns></returns>
        public bool mountCard()
        {
            try
            {
                card.MountFileSystem();
                _isMounted = VolumeInfo.GetVolumes()[0].IsFormatted;
            }
            catch (Exception e)
            {
                _isMounted = false;
                Debug.Print("FEZ SD Card mounting failed: " + e.Message);
            }

            return _isMounted;

        }



        /// <summary>
        /// The function isMounted() will determine if the SD card is mounted or not.
        /// </summary>
        /// <returns></returns>
        public bool isMounted()
        {
            return _isMounted;
        }




        /// <summary>
        /// UnMountCard will unmount the SD card from the device.
        /// </summary>
        public void unMountCard()
        {
            try
            {
                card.UnmountFileSystem();
                _isMounted = false;
            }
            catch (Exception e)
            {
                Debug.Print("Unmounting the card failed: " + e.Message);
            }
        }



        /// <summary>
        /// openFile will open a certain file for writing.
        /// </summary>
        /// <param name="name">The filename</param>
        public void openFile(String name)
        {
            try
            {
                string rootDirectory = VolumeInfo.GetVolumes()[0].RootDirectory;
                FileHandle = new FileStream(rootDirectory + @"\" + name, FileMode.Append);
            }
            catch (Exception e)
            {
                Debug.Print("Error while opening the file: " + e.Message);
            }
        }



        /// <summary>
        /// writeFile will write the given String to the file.
        /// </summary>
        /// <param name="name">The filename</param>
        public void writeFile(String text)
        {
            if (FileHandle != null)
            {
                try
                {
                    byte[] data = Encoding.UTF8.GetBytes(text);
                    FileHandle.Write(data, 0, data.Length);
                }
                catch (Exception e)
                {
                    
                }
            }
        }



        /// <summary>
        /// writeFile will write the given String to the file.
        /// </summary>
        /// <param name="name">The filename</param>
        public void closeFile()
        {
            try
            {
                if (FileHandle != null) FileHandle.Close();
            }
            catch (Exception e)
            {
                Debug.Print("Error while closing the file: " + e.Message);
            }        
        }


        /// <summary>
        /// Dispose() will close the card object.
        /// </summary>
        public void Dispose()
        {
            if (_isMounted)
            {
                _isMounted = false;
                if (card != null) card.Dispose();
                card = null;
            }
        }
    }

}




