using System;
using Microsoft.SPOT;


/// <summary>
/// The CmdPanicButton will set or clear a group of I/O output channels. You activate the function by pressing
/// the button for 3 seconds.
/// </summary>
class CmdPanicButton : SwitchButton 
{
    // variables
    private static int _PRESSING_TIME = 3000;       // pressing time to activate the function, in milliseconds.
    private bool panicButtonEnabled = false;        // current state of the button
    private bool buttonReleasedAgain = true ;       // is the button released again?
    private int[] _channelsToChange;                // array containing the channels to change


    /// <summary>
    /// This parameter allows you to set the time the button has to be pressed.
    /// </summary>
    public static int PRESSING_TIME
    {
        get { return _PRESSING_TIME; }
        set { _PRESSING_TIME = value; }
    }




    /// <summary>
    /// The constructor CmdPanicButton is called upon the creation of the class.
    /// </summary>
    /// <param name="inputChannel">The input channel to monitor.</param>
    /// <param name="channelsToChange">An array containing the channel numbers that must
    /// be included in the action. You don't have to pass an array to the function: you
    /// can just pass a series of integer channel numbers, separate by commas.</param>
    public CmdPanicButton(int inputChannel, params int[] channelsToChange)
    {
        _channelsToChange = channelsToChange; 
        createPermanentInstance(this, inputChannel);
    }



    /// <summary>
    /// buttonPressed() is called when the user keeps pressing the button. 
    /// </summary>
    /// <param name="time_ms"></param>
    public override void buttonPressed(int time_ms) 
    {
        if (time_ms > PRESSING_TIME)
        {
            if (buttonReleasedAgain)
            {
                buttonReleasedAgain = false;
                panicButtonEnabled = !panicButtonEnabled;

                if (panicButtonEnabled)
                    setGroup();
                else
                    clearGroup();
            }
        }
    }



    /// <summary>
    /// buttonUnpressed() is called when the user releases the button. We need to catch
    /// this event, otherwise the outputs will invert if the user keeps pressing the button.
    /// </summary>
    public override void buttonUnpressed() {
        buttonReleasedAgain = true; 
    }



    /// <summary>
    /// setGroup() will set the specified I/O channels.
    /// </summary>
    public void setGroup()
    {
        foreach (int channel in _channelsToChange )
            k8000().setIOchannel(channel);
    }



    /// <summary>
    /// clearGroup() will clear the specified I/O channels.
    /// </summary>
    public void clearGroup()
    {
        foreach (int channel in _channelsToChange)
            k8000().clearIOchannel(channel);
    }
}

