using GHIElectronics.NETMF.Hardware;
using GHIElectronics.NETMF.Net;
using GHIElectronics.NETMF.Net.NetworkInformation;
using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using System.IO;
using GHIElectronics.NETMF.Net.Sockets;


/* The class CmdRTC provides a way to obtain the current time from an NTP (Network Time Protocol) server.
 * This time is synchronized every X hours.
 * 
 * The class uses a background thread to avoid the Home Automation program is blocked while communications
 * occur.
 * 
 * 
 * Remark:
 * All FEZ network functionality is coded in this class. This means this should be the only class with
 * network functionality. You can do otherwise but you would have to move all networking code to a 
 * separate class.
 */
class CmdRTC : SwitchButton 
{
    // Default network settings. They should be redefined through the constructors of the class.
    private static byte[] _ip= { 192, 168, 2, 150 };
    private static byte[] _subnet = { 255, 255, 255, 0 };
    private static byte[] _gateway = { 192, 168, 2, 1 };
    private static byte[] _mac = { 0x00, 0x88, 0x98, 0x90, 0xD4, 0xE0 };
    private static byte[] _ntpHost = {195,130,132,18};
    private static byte[] _dns = { 8,8,8,8 };
    private static int _timeOffset = 0;

    // the firstRun variable makes sure the NTP is always synchronized when the FEZ is first powered.
    private bool firstRun = true;

    private int _updateInterval=-1;
    private long _lastUpdate = -1;

    // The NTPthread object referes to the background thread. 
    private Thread NTPthread;



    #region "Properties"
    /// <summary>
    /// IP is the fixed IP address of your FEZ.
    /// </summary>
    public static byte[] ip
    {
        get { return _ip; }
        set { _ip = value; }
    }


    /// <summary>
    /// SUBNET is the subnet mask of your network
    /// </summary>
    public static byte[] subnet
    {
        get { return _subnet; }
        set { _subnet = value; }
    }


    /// <summary>
    /// GATEWAY is the gateway of your network (usually the IP address of the router)
    /// </summary>
    public static byte[] gateway
    {
        get { return _gateway; }
        set { _gateway = value; }
    }



    /// <summary>
    /// MAC is the MAC address your FEZ should get. Each device on your LAN should have
    /// a unique hexadecimal MAC address.
    /// </summary>
    public static byte[] mac
    {
        get { return _mac; }
        set { _mac = value; }
    }


    /// <summary>
    /// NTPHOST is the IP address of the NTP server.
    /// </summary>
    public static byte[] ntpHost
    {
        get { return _ntpHost; }
        set { _ntpHost = value; }
    }


    /// <summary>
    /// DNS is the IP address of your DNS server. You could use 8.8.8.8 (Google's DNS server).
    /// It doesn't matter too much: the FEZ doesn't use DNS in this application.
    /// </summary>
    public static byte[] dns
    {
        get { return _dns; }
        set { _dns = value; }
    }


    public static int timeOffset
    {
        get { return _timeOffset; }
        set { _timeOffset = value; }
    }

    #endregion






    /// <summary>
    /// Constructor for the class.
    /// </summary>
    /// <param name="updateInterval">The interval you want to use to synchronize the RTC clock (hours).
    /// If the FEZ is powered constantly you can usually set an interval of serveral days/weeks.</param>
    public CmdRTC (int updateInterval) 
    {
        if (updateInterval < 1 || updateInterval > 23) throw new Exception("Please choose an RTC update interval between 1 and 23 hours)");
        _updateInterval=updateInterval;
        activateNetwork();                  // activate the network
       
        // read the actual RTC time from the network
        setTimeFromRTC();

        // here we will setup a new thread that reads the NTP in the background
        NTPthread = new Thread(new ThreadStart(readNTPtime));
        NTPthread.Priority = ThreadPriority.Lowest;

        createPermanentInstance(this);
    }




    /// <summary>
    /// showThreadState() has been written for Debugging purposes. Currently it is not called from
    /// the program.
    /// </summary>
    private void showThreadState() {
        switch (NTPthread.ThreadState) {
            case ThreadState.AbortRequested:
                Debug.Print ("ThreadState.AbortRequested");
                break;

            case ThreadState.Background:
                Debug.Print ("ThreadState.Background");
                break;

            case ThreadState.Running:
                Debug.Print ("ThreadState.Running");
                break;

            case ThreadState.Stopped:
                Debug.Print ("ThreadState.Stopped");
                break;

            case ThreadState.StopRequested:
                Debug.Print ("ThreadState.StopRequested");
                break;

            case ThreadState.Suspended:
                Debug.Print ("ThreadState.Suspended");
                break;

            case ThreadState.SuspendRequested:
                Debug.Print ("ThreadState.SuspendRequested");
                break;

            case ThreadState.Unstarted:
                Debug.Print ("ThreadState.Unstarted");
                break;

            case ThreadState.WaitSleepJoin:
                Debug.Print ("ThreadState.WaitSleepJoin");
                break;
        }
    }




    /// <summary>
    /// activateNetwork() will activate the network. This method should be called from the constructor only.
    /// </summary>
    private void activateNetwork() 
    {
        WIZnet_W5100.ReintializeNetworking();
        WIZnet_W5100.Enable(SPI.SPI_module.SPI1, (Cpu.Pin)FEZ_Pin.Digital.Di10, (Cpu.Pin)FEZ_Pin.Digital.Di7, true);
        NetworkInterface.EnableStaticIP(ip, subnet,gateway, mac);
        NetworkInterface.EnableStaticDns(dns);
    }




    /// <summary>
    /// buttonCycle is executed during each cycle. Here we will try to determine if it is time
    /// to update the RTC clock.
    /// </summary>
    public override void buttonCycle() 
    {

        try
        {
            TimeSpan elapsedSpan = new TimeSpan(DateTime.Now.Ticks - _lastUpdate);

            if (elapsedSpan.Hours >= _updateInterval || firstRun==true)
            {
                _lastUpdate = DateTime.Now.Ticks;
                firstRun = false;

                switch (NTPthread.ThreadState)
                {
                    case ThreadState.Running:
                        // the thread is already running. This is an indication the 
                        // NTP update interval is too short as this should never
                        // occur.
                        Debug.Print("Error: NTP update interval too short");
                        break;

                    case ThreadState.Stopped:
                        // The thread was started before and is now stopped
                        Debug.Print("NTP Background thread restarted");
                        NTPthread = new Thread(new ThreadStart(readNTPtime));
                        NTPthread.Priority = ThreadPriority.Lowest;
                        NTPthread.Start();
                        break;

                    case ThreadState.Unstarted:
                        // The Thread has never been started before        
                        Debug.Print("NTP Background thread started");
                        NTPthread.Start();
                        break;
                }
            }
        }
        catch (Exception e)
        {
            Debug.Print("An error occured while attemping to read the NTP time: " + e.Message);
        }


    }




    /// <summary>
    /// setTimeFromRTC() will set the time from the RTC clock, but only if the data from the RTC
    /// appears to be valid. For this we will check if the year is higher than 2011.
    /// </summary>
    private static void setTimeFromRTC()
    {
        DateTime rtcTime = RealTimeClock.GetTime();
        if (rtcTime.Year > 2011)
        {
            Debug.Print("Local time was read from the RTC: " + RealTimeClock.GetTime().ToString());
            Utility.SetLocalTime(RealTimeClock.GetTime());
        }
    }




    /// <summary>
    /// readNTPtime() will read time using the SNTP protocol. For this it will send an UDP packet
    /// to a NTP server. To minimize the impact on our Home Automation program, we run this request
    /// in a background thread.
    /// </summary>
    private void readNTPtime()
    {
        Debug.Print("Thread running ...");
        DateTime retrievedTime = getSNTPtime(ntpHost, _timeOffset);
        if (retrievedTime != DateTime.MinValue)
        {
            Utility.SetLocalTime(retrievedTime);
            RealTimeClock.SetTime(retrievedTime);
            Debug.Print("RTC adjusted: " + retrievedTime.ToString());
        }
    }



    /// <summary>
    /// getSNTPtime will get the time through the NTP protocol. 
    /// </summary>
    /// <param name="ntpServer">The IP address of the NTP server</param>
    /// <param name="timeOffsetSeconds">The offset time</param>
    /// <returns></returns>
    public DateTime getSNTPtime(byte[] ntpServer, int timeOffsetSeconds)
    {
        Debug.Print("getSNTPtime Time called");
        Socket ntpSocket = null;
        try
        {
            EndPoint serverSide = new IPEndPoint(new IPAddress(_ntpHost), 123);
            ntpSocket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);

            // Construct NTP protocol request packet. 
            // We fill the packet with zeroes. Only the first byte is set to 1B.
            byte[] ntpData = new byte[48];
            for (int i = 1; i < 48; i++) ntpData[i] = 0;
            ntpData[0] = 0x1B;  // set version 4

            // Send request
            ntpSocket.SendTo(ntpData, serverSide);

            // Wait for a response
            // The timeout is 30 seconds.
            if (ntpSocket.Poll(30000000 , SelectMode.SelectRead))
            {
                // Read the received SNTP data. This reply should have 48 bytes.
                ntpSocket.ReceiveFrom(ntpData, ref serverSide);
                ntpSocket.Close();

                // The returned data is a 64-bit unsigned fixed-point number. It contains the number
                // of seconds since 01/01/1900 00:00, with a resolution of 0.2 picoseconds. 
                // The first 32 bits are the integer part. The last 32 bits are the fraction part.
                //
                // We first try to read the integer and fraction part from the floating point.
                byte offsetTransmitTime = 40;
                ulong integer = 0;
                ulong fraction = 0;
                for (int i = 0; i <= 7; i++)
                {
                    if (i<4)
                        integer = 256 * integer + ntpData[offsetTransmitTime + i];
                    else
                        fraction = 256 * fraction + ntpData[offsetTransmitTime + i];
                }

                // now we put the calculated values into an unsigned long variable.
                ulong msSince1900;
                msSince1900 = integer * 1000 + (fraction * 1000) / 100000000;

                // The resulted variable contains the number of milliseconds since 01/01/1900. We now will
                // add this number to 01/01/1900 to get a DateTime object with the calculated time.
                DateTime ntpTime = new DateTime(1900, 1, 1) + TimeSpan.FromTicks((long)msSince1900 * 10000);

                // The given time is the GMT time. We need to add a certain offset to calculate it
                // for another timezone. This time is given in seconds.
                return ntpTime.AddSeconds(timeOffsetSeconds);
            }
            else
            {
                Debug.Print("UDP reading time-out (no response to the request)!");
                ntpSocket.Close();
            }
        }
        catch (Exception e)
        {
            try
            {
                Debug.Print("NTP Exception! " + e.Message);
                ntpSocket.Close();
            }
            catch { }
        }

        return DateTime.MinValue;
    }
}

