using System;
using Microsoft.SPOT;


/// <summary>
/// The class CmdSecurity provides security for your house. It will let lights go on and of automatically.
/// The functions in this class can be customized completely.
/// </summary>
class CmdSecurity : SwitchButton 
{

    private static int _MINIMAL_BURN_TIME_sec = 5 * 60;       // the minimal time a light should be on (seconds)
    private static int _MAXIMAL_BURN_TIME_sec = 60 * 60;      // the maximal time a light should be on (seconds)
    private static int _MINIMAL_PRESS_TIME_ms = 1000;        // the time you need to press the buton to activate the function
    private static int _RANDOM_CHANCE = 1000;                 // the chance an output will become active (chance = 1/RANDOM_CHANCE)

    // internal declarations
    private bool _alwaysOneActive = false;                   // should there always be one channel active?
    private bool _powerSavingMode = false;                   // power saving active? (max one channel active
    private long previousTime = DateTime.Now.Ticks;
    private int[] _channelList;                              // array with the channel numbers
    private int[] _randomTimes;                              // array with the activation data for each channel.
    private bool securityEnabled = false;                     // security system activated?
    private bool waitForRelease = false;                     // wait for the button to release?



    /// <summary>
    /// This parameter allows you to set the time in which the dimmer is dimmed from max to zero.
    /// </summary>
    public static int MINIMAL_BURN_TIME_sec
    {
        get { return _MINIMAL_BURN_TIME_sec; }
        set { _MINIMAL_BURN_TIME_sec = value; }
    }


    /// <summary>
    /// This parameter allows you to set the time in which the dimmer is dimmed from max to zero.
    /// </summary>
    public static int MAXIMAL_BURN_TIME_sec
    {
        get { return _MAXIMAL_BURN_TIME_sec; }
        set { _MAXIMAL_BURN_TIME_sec = value; }
    }


    /// <summary>
    /// This parameter allows you to set the time in which the dimmer is dimmed from max to zero.
    /// </summary>
    public static int MINIMAL_PRESS_TIME_ms
    {
        get { return _MINIMAL_PRESS_TIME_ms; }
        set { _MINIMAL_PRESS_TIME_ms = value; }
    }


    /// <summary>
    /// This parameter allows you to set the time in which the dimmer is dimmed from max to zero.
    /// </summary>
    public static int RANDOM_CHANCE
    {
        get { return _RANDOM_CHANCE; }
        set { _RANDOM_CHANCE = value; }
    }


    /// <summary>
    /// CmdSecurity will initialize the object.
    /// </summary>
    /// <param name="inputChannel">the button to activate/deactivate the function.</param>
    /// <param name="alwaysOneActive">should there always be one light activated?</param>
    /// <param name="powerSavingMode">power saving mode: activate no more than 1 light</param>
    /// <param name="channelsToSecure">A list of the channels to include in the action</param>
    public CmdSecurity(int inputChannel, bool alwaysOneActive, bool powerSavingMode, params int[] channelsToSecure)
    {
        _alwaysOneActive = alwaysOneActive;
        _powerSavingMode = powerSavingMode;
        _channelList = channelsToSecure;
        _randomTimes = new int[channelsToSecure.Length];

        // clear all random times
        for (int i = 0; i < _randomTimes.Length; i++)
            _randomTimes[i] = 0;

        createPermanentInstance(this, inputChannel);
    }




    /// <summary>
    /// buttonPressed() is called when the user pressed a button. The function will determine
    /// if the user kept pressing the button for x seconds.
    /// </summary>
    /// <param name="time_ms"></param>
    public override void buttonPressed(int time_ms) 
    {
        if (time_ms > _MINIMAL_PRESS_TIME_ms && !waitForRelease)
        {
            if (securityEnabled)
                securityEnabled = false;
            else
            {
                securityEnabled = true;
            }

            waitForRelease = true;
        }
    }



    /// <summary>
    /// buttonUp() sets the push lock to false when the button is released.
    /// </summary>
    public override void buttonUp() 
    {
        waitForRelease = false;
    }



    /// <summary>
    /// buttonCycle() is executed multiple times per second. It will set/clear the selected channels. 
    /// </summary>
    public override void buttonCycle()
    {
        if (securityEnabled)
        {   // only perform actions when the security is enabled.

            // first we verify if there is one channel active, if required.
            if (_alwaysOneActive)
            {   
                if (countActiveChannels() == 0)
                {   // no channels are active. We should set one.
                    int selectedChannel = getRandomChannel();
                    int randomTime = getNewRandomTime();
                    _randomTimes[selectedChannel] = randomTime;
                    k8000().setIOchannel(_channelList[selectedChannel]);
                    //Debug.Print("FEZ AlwaysOneActive: setting channel " + _channelList[selectedChannel] + " for " + randomTime + " seconds");
                }
            }

            // if power saving is off, we can optionally add more channels. First we need to specify if a channel should be started.
            // the chance a channel is started, is 1:1000.
            if (_powerSavingMode==false)
            {
                if (isLuckyLoop())
                {   // a chance of 1:1000 struck: we can activate the channel.
                    int selectedChannel = getRandomChannel();
                    int randomTime = getNewRandomTime();
                    _randomTimes[selectedChannel] = randomTime;
                    k8000().setIOchannel(_channelList[selectedChannel]);
                    //Debug.Print("FEZ NoPowerSaving: setting channel " + _channelList[selectedChannel] + " for " + randomTime + " seconds");
                }
            }


            // now we extract the elapsed times from the arrays (expressed in seconds)
            long elapsedTimeInMs = (DateTime.Now.Ticks - previousTime) /10000;
            if (elapsedTimeInMs > 1000)
            {
                int elapsedTimeInSeconds = (int)(elapsedTimeInMs / 1000);

                for (int i = 0; i < _randomTimes.Length; i++)
                {
                    if (_randomTimes[i] > 0)
                    {
                        if (_randomTimes[i] > 0) _randomTimes[i] -= elapsedTimeInSeconds;

                        if (_randomTimes[i] <= 0)
                        {
                            _randomTimes[i] = 0;
                            k8000().clearIOchannel(_channelList[i]);
                            //Debug.Print("FEZ elapsedCheck: clearing channel " + _channelList[i]);
                        }
                    }




                }
                previousTime = DateTime.Now.Ticks;
            }            
        }      
    }



    /// <summary>
    /// isLuckyLoop() will return true if a channel can be activated. It draws a random number between 1 and
    /// RANDOM_CHANCE. You should chose the value of RANDOM_CHANCE with care. The loop is repeated a few times
    /// per second, or about 10.000 times per hour.
    /// </summary>
    /// <returns></returns>
    private bool isLuckyLoop() 
    {
        Random s = new Random();
        int chance = s.Next(RANDOM_CHANCE);
        return (chance == 1);
    }



    /// <summary>
    /// getNewRandomTime()
    /// </summary>
    /// <returns></returns>
    private int getNewRandomTime()
    {
        Random s = new Random();
        int randomTime = s.Next(_MAXIMAL_BURN_TIME_sec - _MINIMAL_BURN_TIME_sec) + _MINIMAL_BURN_TIME_sec;
        return randomTime;
    }




    /// <summary>
    /// getRandomChannel() will generate the index of a random channel to activate.
    /// </summary>
    /// <returns></returns>
    private int getRandomChannel()
    {
        Random s = new Random();
        int randomIndex = s.Next(_channelList.Length);
        return randomIndex;
    }




    /// <summary>
    /// countActiveChannels() will count the channels that are currently activated.
    /// </summary>
    /// <returns></returns>
    private int countActiveChannels()
    {
        int countedChannels = 0;

        foreach (int channel in _randomTimes)
            if (channel > 0) countedChannels++;

        return countedChannels;
    }







}
