using System;
using Microsoft.SPOT;


/// <summary>
/// The CmdTimedEvents class allows you to execute certain event at certain moments in time.
/// </summary>
class CmdTimedEvents : SwitchButton 
{
    // declarations.
    private TimedEvents eventsCollection;
    private String lastTime="";

    /// <summary>
    /// CmdTimedEvents is the constructor of the class
    /// </summary>
    /// <param name="theEvents">An TimedEvents object filled with the desired events</param>
    public CmdTimedEvents(TimedEvents theEvents)
    {
        eventsCollection = theEvents;
        createPermanentInstance(this);
    }


    /// <summary>
    /// getTime() returns a String with the time in HH:MM format.
    /// </summary>
    /// <returns>A String with the time.</returns>
    public String getTime()
    {
        String hours = DateTime.Now.Hour.ToString();
        String minutes = DateTime.Now.Minute.ToString();
        if (hours.Length == 1) hours = "0" + hours;
        if (minutes.Length == 1) minutes = "0" + minutes;
        return (hours + ":" + minutes);
    }



    /// <summary>
    /// buttonCycle() is performed during each cycle of the K8000.
    /// </summary>
    public override void buttonCycle() 
    {
        // first calculate the current time (String format)
        String timeString = getTime();

        // this check will make sure an event won't be executed multiple 
        // times during the same minute.
        if (lastTime != timeString)
        { 
            // loop through all events once per minute. Then check the state.
            for (int i = 0; i < eventsCollection.countEvents(); i++)
            {
                // loop through all known events
                if (eventsCollection.getTime(i) == timeString)
                {   // the current time matches.
                    if ((eventsCollection.getState(i)) == false)
                        k8000().clearIOchannel(eventsCollection.getIO(i));
                    else
                        k8000().setIOchannel(eventsCollection.getIO(i));
                }
            }

            // set the lastTime variable
            lastTime = timeString;
        }
    }
}




/// <summary>
/// The TimedEvents class represents an object with a collection of other TimedEvent objects.
/// </summary>
class TimedEvents
{
    private TimedEventObject[] myEvents;


    /// <summary>
    /// TimedEvents() is the constructor. It will initialize the class.
    /// </summary>
    public TimedEvents()
    {
        myEvents = new TimedEventObject[0];
    }


    /// <summary>
    /// add() will add a new timed event to the class.
    /// </summary>
    /// <param name="time">the time in HH:MM format</param>
    /// <param name="IO">the I/O channel</param>
    /// <param name="state">the desired state (ON,OFF  > true/false)</param>
    public void add(string time, int IO, bool state)
    {
        TimedEventObject myEvent = new TimedEventObject();
        myEvent.time = time;
        myEvent.IO = IO;
        myEvent.state = state;
        addToArray(myEvent);
    }


    /// <summary>
    /// getTime() gets the time for the given index.
    /// </summary>
    /// <param name="index">The index of the desired item</param>
    /// <returns></returns>
    public String getTime(int index) 
    {
        return myEvents[index].time;
    }



    /// <summary>
    /// getIO() gets the I/O channel for the given index.
    /// </summary>
    /// <param name="index">The index of the desired item</param>
    /// <returns></returns>
    public int getIO(int index) 
    {
        return myEvents[index].IO;
    }



    /// <summary>
    /// getState() gets the state for the given index.
    /// </summary>
    /// <param name="index">The index of the desired item</param>
    /// <returns></returns>
    public bool getState(int index) 
    {
        return myEvents[index].state;
    }



    /// <summary>
    /// countEvents() gets the state for the given index.
    /// </summary>
    /// <param name="index">The index of the desired item</param>
    /// <returns></returns>
    public int countEvents()
    {
        return myEvents.Length;
    }




    /// <summary>
    /// The method addToArray() will add a new object to the TimedEvents class.
    /// </summary>
    /// <param name="myEvent">The event to add.</param>
    private void addToArray(TimedEventObject myEvent)
    {
        int arraySize = myEvents.Length;
        TimedEventObject[] tempArray = new TimedEventObject[arraySize+1];
        if (myEvents != null)
        {
            System.Array.Copy(myEvents, tempArray, System.Math.Min(myEvents.Length, tempArray.Length));
        }
        myEvents = tempArray;
        myEvents[arraySize] = myEvent;
    }



    /// <summary>
    /// TimedEventObject represents the data structure for a TimedEvent.
    /// </summary>
    internal sealed class TimedEventObject
    {
        private string _time;
        private int _IO;
        private bool _state;

        /// <summary>
        /// The time (String format HH:MM)
        /// </summary>
        public string time
        {
            get { return _time; }
            set { _time = value; }
        }


        /// <summary>
        /// The I/O channel to change (HH:MM)
        /// </summary>
        public int IO
        {
            get { return _IO; }
            set { _IO = value; }
        }


        /// <summary>
        /// The state of the channel (true/false)
        /// </summary>
        public bool state
        {
            get { return _state; }
            set { _state = value; }
        }
    }
}

