using System;
using Microsoft.SPOT;

/// <summary>
/// The CmdTimer class provides a Timer functionality: a certain channel will be made
/// active as soon as another channel has been stopped.
/// </summary>
class CmdTimer : SwitchButton
{
    // declarations
    private int _outputChannel;
    private int _duration;
    private bool _disableTurnOff = false;
    private bool _extendedType;
    private long startTime = -1;           // helper variable: the time (in ticks) when the timer was first started.

    // class behavior constants
    // This constant will define how long it takes before the 
    private static int _DURATION_FOR_SECOND_FUNCTION = 2;


    /// <summary>
    /// This parameter allows you to set the time in which the dimmer is dimmed from max to zero.
    /// </summary>
    public static int DURATION_FOR_SECOND_FUNCTION
    {
        get { return _DURATION_FOR_SECOND_FUNCTION; }
        set { _DURATION_FOR_SECOND_FUNCTION = value; }
    }



    /// <summary>
    /// The constructor CmdTimer is called upon the cration of the class.
    /// </summary>
    /// <param name="inputChannel">The input channel to monitor</param>
    /// <param name="outputChannel">The output channel to control</param>
    /// <param name="duration">The duration of the timer (seconds)</param>
    /// <param name="extendedType">A boolean to indicate if the timer is an extended model or
    /// not. The extended timer has a second function: if the user keeps pushing the button
    /// for x seconds (x is defined in the DURATION_FOR_SECOND_FUNCTION constant), the timer
    /// will be disabled.</param>
    public CmdTimer(int inputChannel, int outputChannel, int duration, bool extendedType)
    {
        _extendedType = extendedType;
        _duration = duration;
        _outputChannel = outputChannel;
        createPermanentInstance(this, inputChannel);
    }

    /// <summary>
    /// The constructor CmdTimer is called upon the cration of the class.
    /// </summary>
    /// <param name="inputChannel">The input channel to monitor</param>
    /// <param name="outputChannel">The output channel to control</param>
    /// <param name="duration">The duration of the timer (seconds)</param>
    /// <param name="extendedType">A boolean to indicate if the timer is an extended model or
    /// not. The extended timer has a second function: if the user keeps pushing the button
    /// for x seconds (x is defined in the DURATION_FOR_SECOND_FUNCTION constant), the timer
    /// will be disabled.</param>
    /// <param name="disableTurnOff">A boolean to indicate you want to be able to
    /// turn off the timer with a second pulse.</param>
    public CmdTimer(int inputChannel, int outputChannel, int duration, bool extendedType,bool disableTurnOff)
    {
        _extendedType = extendedType;
        _duration = duration;
        _outputChannel = outputChannel;
        _disableTurnOff = disableTurnOff;
        createPermanentInstance(this, inputChannel);
    }


    /// <summary>
    /// The buttonDown() is executed when the dimmer is just activated. The timer starts, but the countdown
    /// will only begin when the user releases the button first.
    /// </summary>
    /// 
    public override void buttonDown() 
    {
        if (startTime > -1)
        {   // there is a timer running
            if (_disableTurnOff == false)
            {
                startTime = -1;
                k8000().clearIOchannel(_outputChannel);
            }
            else
            {
                startTime = DateTime.Now.Ticks;
            }
        }
        else
        {   // there is no timer running or the light is running in continious mode
            k8000().readIOchannel(_outputChannel);

            if (k8000().IO[_outputChannel]==1)
                k8000().clearIOchannel(_outputChannel);
            else
            {
                startTime = DateTime.Now.Ticks;
                k8000().setIOchannel(_outputChannel);
            }
        }
    }




    /// <summary>
    /// The buttonPressed() event is executed when we keep pressing the button. It will
    /// evaluate if the user keeps pressing the button.
    /// </summary>
    /// <param name="time_ms"></param>
    public override void buttonPressed(int time_ms) 
    {
        if (_extendedType == true)
        {   // disable the timer when the button is pressed for a certain time, IF extended timer is enabled
            if (time_ms > 1000 * DURATION_FOR_SECOND_FUNCTION)
            {
                startTime = -1;
            }
                
        }
    }


    /// <summary>
    /// buttonUnpressed() is executed when the button isn't pressed. 
    /// </summary>
    public override void buttonUnpressed()
    {
        if (startTime > -1)
        {
            long elapsedTime = (DateTime.Now.Ticks - startTime) / (10000 * 1000);

            if (elapsedTime > _duration)
            {   // timer elapsed.
                k8000().clearIOchannel(_outputChannel);
                startTime = -1;
            }
        }
    }
}
