using System;
using Microsoft.SPOT;

/// <summary>
/// SwitchButton is an abstract class. The other function classes inherit its properties.
/// I created the class to share the K8000 object very easily.
/// </summary>
abstract class SwitchButton
{

    #region "Static Methods"


    // internal _myInterface variable: contains the reference to the K8000 object (MUST be set).
    private static Velleman_K8000 _myInterface;
    private static int _numberOfCards;

    // internal variable containing an array of this object.
    private static SwitchButton[] objectReferences;


    // The numberOfK8000Cards property will initialize the number of K8000 cards.
    public static int numberOfK8000Cards
    {
        get { return _numberOfCards; }
        set 
        { 
            _numberOfCards = value;
            _myInterface = new Velleman_K8000(_numberOfCards, false);
            _myInterface.configAllIOasOutput(); 
            }
    }


    /// <summary>
    /// The static constructor SwitchButton() is loaded when the class is used for the
    /// very first time. It will initialize the objectReferences array.
    /// </summary>
    static SwitchButton()
    {   // First time initialisation.
        objectReferences = new SwitchButton[0];
    }



    /// <summary>
    /// The static method add() will add a new object reference to the array with references.
    /// </summary>
    /// <param name="objectReference">New objectReference to add. It can be any of the Cmd classes.</param>
    public static void add(SwitchButton objectReference)
    {
        int arraySize = objectReferences.Length;
        SwitchButton[] tempArray = new SwitchButton[arraySize+1];
        if (objectReferences != null)
        {
            System.Array.Copy(objectReferences, tempArray, System.Math.Min(objectReferences.Length, tempArray.Length));
        }
        objectReferences = tempArray;
        objectReferences[arraySize] = objectReference;
    }



    /// <summary>
    /// Will actually execute any of the created commands.
    /// </summary>
    public static void executeAll()
    {
        while (true)
        {   // perform an endless loop
            for (int i = 0; i < objectReferences.Length; i++)
            {   // for each of the stored object references.
                try
                {
                    objectReferences[i].doAction();
                }
                catch (Exception e)
                {   // an exception occured. Just skip to the next object.
                    Debug.Print("FEZ execution error: " + e.Message);
                }
            }
        }
    }



    // interfaceObject property to assign the K8000 object.
    public static Velleman_K8000 interfaceObject
    {
        get { return _myInterface; }
        set { _myInterface = value; }
    }
    #endregion





    #region "Master Object Methods"

    // declaration of the variables
    private int _buttonChannel = 0;         // the selected channel the button is connected to.      
    private int previousButtonState = 0;    // the previous state of the selected channel's button.
    private long buttonPressMoment = -1;    // the exact time a button was first pressed.


    /// <summary>
    /// The function createPermanentInstance() will add the object of the Cmd... class to the static
    /// array of the SwitchButton class. You are REQUIRED to call this function from each constructor
    /// in the Cmd... class.
    /// </summary>
    /// <param name="objectReference"></param>
    /// <param name="inputChannel"></param>
    public void createPermanentInstance(SwitchButton objectReference, int inputChannel)
    {
        _buttonChannel = inputChannel;
        k8000().configIOchannelAsInput(inputChannel);

        int arraySize = objectReferences.Length;
        SwitchButton[] tempArray = new SwitchButton[arraySize + 1];
        if (objectReferences != null)
        {
            System.Array.Copy(objectReferences, tempArray, System.Math.Min(objectReferences.Length, tempArray.Length));
        }
        objectReferences = tempArray;
        objectReferences[arraySize] = objectReference;
    }




    /// <summary>
    /// The function createPermanentInstance() will add the object of the Cmd... class to the static
    /// array of the SwitchButton class. You are REQUIRED to call this function from each constructor
    /// in the Cmd... class.
    /// </summary>
    /// <param name="objectReference"></param>
    /// <param name="inputChannel"></param>
    public void createPermanentInstance(SwitchButton objectReference)
    {
        _buttonChannel = -1;

        int arraySize = objectReferences.Length;
        SwitchButton[] tempArray = new SwitchButton[arraySize + 1];
        if (objectReferences != null)
        {
            System.Array.Copy(objectReferences, tempArray, System.Math.Min(objectReferences.Length, tempArray.Length));
        }
        objectReferences = tempArray;
        objectReferences[arraySize] = objectReference;
    }

    


    /// <summary>
    /// The function k8000() will return a reference to the K8000 method. The method is accessible
    /// from the classes that inherit from this class.
    /// </summary>
    /// <returns></returns>
    public Velleman_K8000 k8000()
    {
        return _myInterface;
    }




    /// <summary>
    /// doAction() performs the actual action of the input button. It will poll the current state of the 
    /// input button, and generate the required events. The events are methods in this class, but they
    /// should be overridden by the child classes.
    /// </summary>
    public void doAction()
    {   // first poll the current state of the channel.

        if (_buttonChannel > 0)
        {
            k8000().readIOchannel(_buttonChannel);

            // Then evaluate if it is on or off.
            if (k8000().IO[_buttonChannel] == 1)
            {   // POSSIBILITY A: THE BUTTON IS PRESSED.
                if (previousButtonState == 0)
                {   // POSSIBILITY A.1: THE BUTTON IS PRESSED AND WAS NOT PRESSED BEFORE --> BUTTON DOWN EVENT.
                    buttonPressMoment = DateTime.Now.Ticks;
                    buttonDown();
                }
                else
                {   // POSSIBILITY A.2: THE BUTTON IS PRESSED BUT WAS PRESSED BEFORE --> BUTTON PRESSED EVENT.
                    long buttonPressTimeLong = (DateTime.Now.Ticks - buttonPressMoment) / 10000;
                    if (buttonPressTimeLong < 2147483647)
                    {
                        int buttonPressTime = (int)buttonPressTimeLong;
                        buttonPressed(buttonPressTime);
                    }



                }
            }
            else
            {   // POSSIBILITY B: THE BUTTON IS UNPRESSED.
                if (previousButtonState == 0)
                {   // POSSIBILITY B.1: THE BUTTON IS NOT PRESSED AND WAS NOT PRESSED BEFORE --> BUTTON UNPRESSED EVENT.
                    buttonUnpressed();
                }
                else
                {   // POSSIBILITY B.2: THE BUTTON IS NOT PRESSED BUT WAS PRESSED BEFORE --> BUTTON UP EVENT.
                    buttonPressMoment = -1;
                    buttonUp();
                }
            }

            // set the state that is to be read in the next cycle.
            previousButtonState = k8000().IO[_buttonChannel];

            // call an optional event that is executed anyway.
            buttonCycle();

        }
        else
        {
            // call an event that is executed each time.
            buttonCycle();
        }



    }
    #endregion





    #region "Event Processing & Methods"
    /******************************************************************************************
     * This section of the code contains the actual headers. If you need any of them, you
     * can override them in any of the child classes.
     * ****************************************************************************************/
    /// <summary>
    /// buttonDown() is called when the button is first pressed: when a state change from "0" to "1"
    /// is detected.
    /// </summary>
    public virtual void buttonDown() { }

    /// <summary>
    /// buttonUp() is called when the button is first unpressed: when a state change from "1" to "0"
    /// is detected.
    /// </summary>
    public virtual void buttonUp() { }

    /// <summary>
    /// buttonPressed() is called when the button is currently pressed. The pressing time is passed 
    /// as a parameter.
    /// </summary>
    /// <param name="time_ms">The time the button has been pressed (milliseconds)</param>
    public virtual void buttonPressed(int time_ms) { }

    /// <summary>
    /// buttonPressed() is called when the button is not pressed. The pressing time is not passed here,
    /// this should be unnecessary and could cause an overflow error when time gets too high.
    /// </summary>
    public virtual void buttonUnpressed() { }

    /// <summary>
    /// buttonCycle() is during each cycle, no matter what the current state of the buttons is. I
    /// introduced this event for the really special cases. You shouldn't need it often.
    /// </summary>
    public virtual void buttonCycle() { }


    /// <summary>
    /// errorReset() occurs when there is a problem with the execution of a certain class. It should
    /// never occur, but you can catch it to reset an object.
    /// </summary>
    public virtual void errorReset() { }

    #endregion

}
